<?php
//
// Docker Config Wizard
// Copyright (c) 2018-2021 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__).'/../configwizardhelper.inc.php');

docker_configwizard_init();

function docker_configwizard_init()
{
    $name = 'docker';
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _('Monitor Docker containers via NCPA or through the cURL API.'),
        CONFIGWIZARD_DISPLAYTITLE => _('Docker'),
        CONFIGWIZARD_FUNCTION => 'docker_configwizard_func',
        CONFIGWIZARD_PREVIEWIMAGE => 'docker.png',
        CONFIGWIZARD_VERSION => '1.1.2',
        CONFIGWIZARD_COPYRIGHT => _("Copyright &copy; 2017-2021 Nagios Enterprises, LLC."),
        CONFIGWIZARD_AUTHOR => _('Nagios Enterprises, LLC.'),
        CONFIGWIZARD_REQUIRES_VERSION => 550
    );
    register_configwizard($name,$args);
}

// This function is automatically called by the XI wizard framework when the wizard is run
function docker_configwizard_func($mode = '', $inargs = null, &$outargs, &$result)
{
    $wizard_name = 'docker';

    /* Prerequisite software */
    $NCPA_download_url = "https://www.nagios.org/ncpa/#downloads";
    $NCPA_docs_url = "https://www.nagios.org/ncpa/#docs";
    $docker_plugin_url = "/nagiosxi/includes/configwizards/docker/plugins/check_docker.py";
    $docker_docs_url = "https://assets.nagios.com/downloads/nagiosxi/docs/How-to-Monitor-Docker-Containers-with-Nagios-XI.pdf";

    /* Initialize return code and output */
    $result = 0;
    $output = '';
    
    /* Initialize output args */
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {
        case CONFIGWIZARD_MODE_GETSTAGE1HTML:
            /* This defines $nextstep variable, used for determining if the user did not */
            /* provide correct information when the user was on CONFIGWIZARD_MODE_GETSTAGE1HTML */
            $nextstep = encode_form_val(grab_array_var($_POST,'nextstep',false),ENT_QUOTES);

            /* Determine if this is the first wizard run */
            if ($nextstep == '') {
                /* Clear the session array to be sure it is blank */
                unset($_SESSION[$wizard_name]); 
            }

            /* Fresh Wizard Run */
            /* OR */
            /* Continuing from CONFIGWIZARD_MODE_VALIDATESTAGE1DATA due to user error */
            
            if (!isset($_POST['ip_address'])) {
                /* Fresh Wizard Run */
                
                /* Define the session array to hold data from different stages */
                $_SESSION[$wizard_name] = array(); 
                
                /* The following variables need to be defined to prevent coding errors */
                $address = '';
                $socket = '';
                $baseurl = '';
                $port = '5693';
                $token = '';


                $list_type = "none";
                $container_names = array();
                $network_names = array();
                $check_types = array();
            }
            else {
                /* Continuing from CONFIGWIZARD_MODE_VALIDATESTAGE1DATA due to user error */
                
                /* Grab user data already provided from CONFIGWIZARD_MODE_GETSTAGE1HTML */
                /* Prioritize _SESSION, but use inargs if _SESSION is unset. */
                $remote_session = grab_array_var($_SESSION[$wizard_name],'remote',''); // only used in the next line
                $remote = grab_array_var($inargs,'remote', $remote_session);
                $address_session = grab_array_var($_SESSION[$wizard_name],'ip_address','');
                $address = grab_array_var($inargs,'ip_address', $address_session);
                $socket_session = grab_array_var($_SESSION[$wizard_name], 'socket', '');
                $socket = grab_array_var($inargs, 'socket', $socket_session);
                $baseurl_session = grab_array_var($_SESSION[$wizard_name], 'baseurl', '');
                $baseurl = grab_array_var($inargs, 'baseurl', $baseurl_session);
                $port_session = grab_array_var($_SESSION[$wizard_name], 'port', '5693');
                $port = grab_array_var($inargs, 'port', $port_session);
                $token_session = grab_array_var($_SESSION[$wizard_name], 'token', '');
                $token = grab_array_var($inargs, 'token', $port_session);


                $list_type_session = grab_array_var($_SESSION[$wizard_name],'list_type','');
                $list_type = grab_array_var($inargs,'list-type',$list_type_session);

                $container_names_session = grab_array_var($_SESSION[$wizard_name], 'container_names', array());
                $container_names = grab_array_var($inargs, 'containernames', $container_names_session);
                $container_names = array_values(array_filter($container_names));

                $network_names_session = grab_array_var($_SESSION[$wizard_name], 'network_names', array());
                $network_names = grab_array_var($inargs, 'network_names', $network_names_session);
                $network_names = array_values(array_filter($network_names));

                $check_types_session = grab_array_var($_SESSION[$wizard_name], 'check_types', array());
                $check_types = grab_array_var($inargs, 'checktype', $check_types_session);

                $cert_session = grab_array_var($_SESSION[$wizard_name], 'cert', '');
                $cert = grab_array_var($inargs, 'cert', $cert_session);
                $key_session = grab_array_var($_SESSION[$wizard_name], 'key', '');
                $key = grab_array_var($inargs, 'key', $key_session);
                $cacert_session = grab_array_var($_SESSION[$wizard_name], 'cacert', '');
                $cacert = grab_array_var($inargs, 'cacert', $cacert_session);
            }
                
            
            
            /* Now we are creating the HTML for CONFIGWIZARD_MODE_GETSTAGE1HTML */ 

            // Redirect output to buffer -- we can load all 'printed' text into a string by calling ob_get_clean() right before we 'break;' for this case.
            ob_start();
            ?>
            <h5 class="ul"><?php echo _('Docker Server Information'); ?></h5>
            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td class="vt">
                        <label for="remote"><?php echo _('Access Docker via:'); ?> </label>
                    </td>
                    <td>
                        <select name="remote" id="remote" class="form-control">
                            <option value="NCPA" <?php echo is_selected($remote, "NCPA");?>><?php echo _('Remote Agent (NCPA)'); ?></option>
                            <option value="api" <?php echo is_selected($remote, "api");?>><?php echo _('Remote API'); ?></option>
                        </select>
                        <div class="subtext ncpa-only">
                            <?php echo sprintf(_('To use NCPA, make sure to install %1$sthe agent%2$s on your remote machine using %3$sthese instructions%4$s if you haven\'t already. Then, place %5$sthe Docker plugin%6$s in your NCPA plugins folder, following %7$sthis Docker-specific setup documentation.%8$s'), 
                            "<a href=\"" . $NCPA_download_url . "\" target=\"_blank\">", "</a>",
                            "<a href=\"" . $NCPA_docs_url . "\" target=\"_blank\">", "</a>",
                            "<a href=\"" . $docker_plugin_url . "\" target=\"_blank\">", "</a>",
                            "<a href=\"" . $docker_docs_url . "\" target=\"_blank\">", "</a>"); ?>
                        </div>
                    </td>
                </tr>
                <tr>
                    <td class="vt">
                        <label for="ip_address"><?php echo _('IP Address:'); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="ip_address" id="ip_address" value="<?php echo $address; ?>" class="form-control">
                        <div class="subtext"><?php echo _('The IP address or FQDN of the server you would like to monitor.'); ?></div>
                    </td>
                </tr>
                <tr class="ncpa-only">
                    <td class="vt">
                        <label for=""><?php echo _("NCPA Listener Port"); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="port" id="port" value="<?php echo $port; ?>" class="form-control">
                    </td>
                </tr>
                <tr class="ncpa-only">
                    <td class="vt">
                        <label for=""><?php echo _("NCPA Token"); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="token" id="token" value="<?php echo $token; ?>" class="form-control" autocomplete="off">
                    </td>
                </tr>

                <tr class="ncpa-only">
                    <td class="vt">
                        <label for="socket"><?php echo _('Docker Socket:'); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="socket" id="socket" value="<?php echo $socket; ?>" class="form-control">
                        <div class="subtext"><?php echo _('The full path to your docker socket.'); ?> Ex: "/var/run/docker.sock"</div>
                    </td>
                </tr>
                <tr>
                    <td class="vt">
                        <label for="baseurl"><?php echo _('Docker API Base URL:'); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="baseurl" id="baseurl" value="<?php echo $baseurl; ?>" class="form-control">
                        <div class="subtext"><?php echo _('The full URL to your Docker API.'); ?> Ex: "http:/v1.30/" or "http://192.168.1.2:2375/"</div>
                    </td>
                </tr>
            </table>

            <div id="security">
            <h5 class="ul"><?php echo _('Security'); ?></h5>
            <?php echo _('This section is only needed if you\'re using the remote API and have protected your Docker port using TLS.'); ?>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt">
                        <label for="cert"><?php echo _('Certificate:'); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="cert" id="cert" value="<?php echo $cert; ?>" class="form-control">
                        <div class="subtext"><?php echo _('The path to your certificate.'); ?></div>
                    </td>
                </tr>
                <tr>
                    <td class="vt">
                        <label for="key"><?php echo _('Key:'); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="key" id="key" value="<?php echo $key; ?>" class="form-control">
                        <div class="subtext"><?php echo _('The path to your key.'); ?></div>
                    </td>
                </tr>
                <tr>
                    <td class="vt">
                        <label for="cacert"><?php echo _('CA Certification:'); ?></label>
                    </td>
                    <td>
                        <input type="text" size="40" name="cacert" id="cacert" value="<?php echo $cacert;?>" class="form-control">
                        <div class="subtext"><?php echo _('The path to your CA certificate.'); ?></div>
                    </td>
                </tr>
            </table>
            </div>

            <script>
            var canPopulateTables = false;

            $(document).ready(function () {
                $("#list-type").change(function() {
                    $("#network-table").hide();
                    $("#container-table").hide();
                    $("#populate-tables").hide();
                    if($(this).val() === "networks") {
                        $("#network-table").show();
                        if (canPopulateTables) {
                            $("#populate-tables").show();
                        }
                    }
                    else if($(this).val() === "containers") {
                        $("#container-table").show();
                        if (canPopulateTables) {
                            $("#populate-tables").show();
                        }
                    }
                });

                $("#remote").change(function () {
                    $("#populate-tables").hide();
                    canPopulateTables = false;
                    $(".ncpa-only").hide();
                    $("#security").hide();
                    if ($(this).val() === "api") {
                        // using tcp-bound API directly.
                        $("#security").show();
                        if ($("list-type").val() === "networks" || $("#list-type").val() === "containers") {
                            $("#populate-tables").show();                            
                        }
                        canPopulateTables = true;
                    }
                    else { 
                        // using NCPA 
                        $(".ncpa-only").show();
                    }
                });
                if($("#list-type").val() === "networks") {
                    $("#network-table").show();
                }
                else if($("#list-type").val() === "containers") {
                    $("#container-table").show();
                }
                if($("#remote").val() === "api") {
                    $(".ncpa-only").hide();
                    $("#security").show();
                    $("#populate-tables").show();
                }
                else { 
                    // using NCPA
                    $(".ncpa-only").show();
                    $("#security").hide();
                    $("#populate-tables").hide();
                }


                // Table population AJAX/modal
                $("#populate-tables").click(function() {
                    display_child_popup(300);
                    center_child_popup();
                    set_child_popup_content('<div id="popup_header"><b><?php echo _("Please Wait");?></b></div><div id="popup_data"><p></p><ul class="ajaxcommandresult"><li class="commandresultwaiting" id="populate-tables-command-result"><?php echo _("Populating tables...");?> (0/2)</li></ul></div>');

                    // We are in nagiosxi/config/monitoringwizard.php, need to be in nagiosxi/includes/configwizards/docker/table_population.php

                    base_url = "../includes/configwizards/docker/table_population.php";

                    var containers_array = {};
                    containers_array["cert"] = $("#cert").val();
                    containers_array["key"] = $("#key").val();
                    containers_array["cacert"] = $("#cacert").val();
                    containers_array["host"] = $("#baseurl").val();
                    containers_array["mode"] = "containers";

                    var networks_array = $.extend({}, containers_array);
                    networks_array["mode"] = "networks";

                    var global_result = 2;
                    $.post(base_url, networks_array,
                        function(data) {
                            var result = table_inject('network-table', data);
                            mod_global_result(result);
                            display_result(data);
                        },
                        'json'
                    );

                    $.post(base_url, containers_array, 
                        function(data) {
                            var result = table_inject('container-table', data);
                            mod_global_result(result);
                            display_result(data);
                        },
                        'json'
                    );

                    function mod_global_result(result) {
                        if (result === 0) {
                            if (global_result > 0) {
                                global_result -= 1;
                            }
                        }
                        else {
                            global_result = result;
                        }
                    }

                    function display_result(data) {
                        switch (global_result) {
                            case -3: 
                                set_child_popup_content('<div id="popup_header"><b><?php echo _("Error");?></b></div><div id="popup_data"><p></p><ul class="ajaxcommandresult"><li class="commandresulterror" id="populate-tables-command-result"><?php echo _("cURL was called on an incorrect endpoint.");?></li></ul></div>');
                                break;

                            case -2:
                                set_child_popup_content('<div id="popup_header"><b><?php echo _("Error");?></b></div><div id="popup_data"><p></p><ul class="ajaxcommandresult"><li class="commandresulterror" id="populate-tables-command-result"><?php echo _("cURL call did not get a valid response from the listed address.");?></li></ul></div>');
                                break;

                            case -1:
                                set_child_popup_content('<div id="popup_header"><b><?php echo _("Error");?></b></div><div id="popup_data"><p></p><ul class="ajaxcommandresult"><li class="commandresulterror" id="populate-tables-command-result"><?php echo _("Please put a valid URL in the cell labelled \"Docker API Base URL\".");?></li></ul></div>');
                                break;

                            case 0:
                                set_child_popup_content('<div id="popup_header"><b><?php echo _("Success!");?></b></div><div id="popup_data"><p></p><ul class="ajaxcommandresult"><li class="commandresultok" id="populate-tables-command-result"><?php echo _("Both tables populated properly!");?></li></ul></div>');
                                fade_child_popup("green", 3000);
                                break;

                            case 1:
                                set_child_popup_content('<div id="popup_header"><b><?php echo _("Please Wait...");?></b></div><div id="popup_data"><p></p><ul class="ajaxcommandresult"><li class="commandresultwaiting" id="populate-tables-command-result"><?php echo _("Populating tables...");?> (1/2)</li></ul></div>');
                                break;

                            default:
                                set_child_popup_content('<div id="popup_header"><b><?php echo _("Please Wait...");?></b></div><div id="popup_data"><p></p><ul class="ajaxcommandresult"><li class="commandresultwaiting" id="populate-tables-command-result"><?php echo _("Populating tables...");?> (0/2)</li></ul></div>');
                                break;
                        }
                    }
                });

                function table_inject(which_one, data) {
                    if (data === "error_host") {
                        return -1;
                    }
                    else if (data === "error_curl") {
                        return -2; 
                    } else if ("message" in data) {
                        return -3;
                    }

                    var arr_name = "networknames";
                    if (which_one === "container-table") {
                        arr_name = "containernames";
                    }

                    for (x in data) {
                        $("#" + which_one + " .wizard-add-row").click();
                        $("#" + which_one + " > table > tbody > tr > td > input[name='" + arr_name + "[" + x + "]']").val(data[x]);
                    }
                    return 0;
                }
            });
            </script>

            <h5 class="ul"><?php echo _('Checks to run'); ?></h5>
            <?php echo _('These checks can be configured on the next page.');?>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td>
                        <input type="checkbox" class="checkbox" id="containers_exist_box" name="checktype[exist]" <?php echo is_checked(checkbox_binary($check_types["exist"]), "1");?>>
                    </td>
                    <td>
                        <label for="containers_exist_box" class="select-cf-option"><?php echo _('Existing Containers'); ?></label><br>
                        <?php echo _('Determines how many containers exist');?>
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="checkbox" class="checkbox" id="containers_running_box" name="checktype[running]"  <?php echo is_checked(checkbox_binary($check_types["running"]), "1");?>>
                    </td>
                    <td>
                        <label for="containers_running_box" class="select-cf-option"><?php echo _('Running Containers'); ?></label><br>
                        <?php echo _('Determines how many containers are running.'); ?>
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="checkbox" class="checkbox" id="containers_healthy_box" name="checktype[healthy]"  <?php echo is_checked(checkbox_binary($check_types["healthy"]), "1");?>>
                    </td>
                    <td>
                        <label for="containers_healthy_box" class="select-cf-option"><?php echo _('Healthy Containers');?></label><br>
                        <?php echo _('Determines how many containers are healthy.');?>
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="checkbox" class="checkbox" id="containers_cpu_box" name="checktype[cpu]"  <?php echo is_checked(checkbox_binary($check_types["cpu"]), "1"); ?>>
                    </td>
                    <td>
                        <label for="containers_cpu_box" class="select-cf-option"><?php echo _('CPU Usage'); ?></label><br>
                        <?php echo _('Checks the CPU usage of the containers either in aggregate or per container.'); ?>
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="checkbox" class="checkbox" id="containers_memory_box" name="checktype[memory]"  <?php echo is_checked(checkbox_binary($check_types["memory"]), "1"); ?>>
                    </td>
                    <td>
                        <label for="containers_memory_box" class="select-cf-option"><?php echo _('Memory Usage'); ?></label><br>
                        <?php echo _('Checks the memory usage of the containers either in aggregate or per container.'); ?>
                    </td>
                </tr>
                <tr>
                    <td>
                        <label for="list-type"><?php echo _('Monitor'); ?>:</label>
                    </td>
                    <td>
                        <select name="list-type" id="list-type" class="form-control">
                            <option value="none" <?php echo is_selected($list_type, "none");?>><?php echo _('Please select an option.');?></option>
                            <option value="containers" <?php echo is_selected($list_type, "containers");?>><?php echo _('A list of containers');?></option>
                            <option value="networks" <?php echo is_selected($list_type, "networks");?>><?php echo _('The containers on a list of networks');?></option>
                            <option value="all" <?php echo is_selected($list_type, "all");?>><?php echo _('All visible containers');?></option>
                        </select>
                    </td>
                </tr>
                <tr>
                    <td></td>
                    <td valign="middle">
                        <a id="populate-tables" class="btn btn-sm btn-default" style="line-height: 1.1rem; padding: 0.8rem"><?php echo _("Populate Container/Network List");?></a>
                    </td>
                </tr>
            </table>

            <div id="container-table" class="container-table hide">
                <table class="table table-condensed table-no-border table-auto-width adddeleterow" style="margin: 0 0 10px 0;">
                    <tr> 
                        <th><?php echo _('Container Name/ID');?></th>
                    </tr>
            <?php 
            if(is_array($container_names)) {
                foreach($container_names as $index => $container) {
            ?>
                    <tr>
                        <td>
                            <input type="text" size="50" name="containernames[<?php echo $index;?>]" value="<?php echo encode_form_val($container);?>" class="textfield form-control">
                        </td>
                    </tr>
            <?php
                } 
            ?>
                <!-- The 'last blank'. -->
                    <tr>
                        <td>
                            <input type="text" size="50" name="containernames[<?php echo count($container_names);?>]" value="" class="textfield form-control">
                        </td>
                    </tr>
            <?php
            }
            else {
            ?>
                    <tr>
                        <td>
                            <input type="text" size="50" name="containernames[0]" value="" class="textfield form-control">
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <input type="text" size="50" name="containernames[1]" value="" class="textfield form-control">
                        </td>
                    </tr>
            <?php
            }
            ?>
                </table>
            </div>
            <div id="network-table" class="network-table hide">
                <table class="table table-condensed table-no-border table-auto-width adddeleterow" style="margin: 0 0 10px 0;">
                    <tr> 
                        <th><?php echo _('Network Name/ID'); ?></th>
                    </tr>
            <?php 
            if(is_array($network_names)) {
                foreach($network_names as $index => $network) {
            ?>
                    <tr>
                        <td>
                            <input type="text" size="50" name="networknames[<?php echo $index;?>]" value="<?php echo encode_form_val($network);?>" class="textfield form-control">
                        </td>
                    </tr>
            <?php
                } 
            ?>
                    <tr>
                        <td>
                            <input type="text" size="50" name="networknames[<?php echo count($network_names);?>]" value="" class="textfield form-control">
                        </td>
                    </tr>';
            <?php
            }
            else {
            ?>
                    <tr>
                        <td>
                            <input type="text" size="50" name="networknames[0]" value="" class="textfield form-control">
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <input type="text" size="50" name="networknames[1]" value="" class="textfield form-control">
                        </td>
                    </tr>
            <?php
            }
            ?>
                </table>
            </div>
<?php

            $output = ob_get_clean();
            /* The quote and semicolon ends HTML for CONFIGWIZARD_MODE_GETSTAGE1HTML */ 
            /* $output will be passed back to XI framework and rendered to browser */
            /* The next line ends CONFIGWIZARD_MODE_GETSTAGE1HTML */
            break;

            
        /* Form validation for CONFIGWIZARD_MODE_GETSTAGE1HTML */
        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:      
            /* This defines $back variable, used for determining if the Back button */
            /* was clicked when the user was on CONFIGWIZARD_MODE_GETSTAGE2HTML */
            $back = array_key_exists("backButton", $_POST);         
            
            /* If the user came back from CONFIGWIZARD_MODE_GETSTAGE3HTML then we don't need to revalidate and check for errors */
            if ($back) {
                break;
            }
            
            /* Get variables that were passed to us */
                /* NOTE: grab_array_var is an XI wrapper function to fetch values 'if they exist' */
                /* USAGE: grab_array_var($array, $variable, $defaultValue) */
            $remote_session = grab_array_var($_SESSION[$wizard_name],'remote',''); // only used in the next line
            $remote = grab_array_var($inargs,'remote', $remote_session);
            $address_session = grab_array_var($_SESSION[$wizard_name], 'ip_address', '');
            $address = grab_array_var($inargs,'ip_address',$address_session);
            $socket_session = grab_array_var($_SESSION[$wizard_name], 'socket', '');
            $socket = grab_array_var($inargs,'socket',$socket_session);
            $baseurl_session = grab_array_var($_SESSION[$wizard_name], 'baseurl', '');
            $baseurl = grab_array_var($inargs,'baseurl',$baseurl_session);

            $port_session = grab_array_var($_SESSION[$wizard_name], 'port', '');
            $port = grab_array_var($inargs, 'port', $port_session);
            $port = intval($port);
            $token_session = grab_array_var($_SESSION[$wizard_name], 'token', '');
            $token = grab_array_var($inargs, 'token', $token_session);

            $list_type_session = grab_array_var($_SESSION,'list_type','');
            $list_type = grab_array_var($inargs,'list-type',$list_type_session);

            $container_names_session = grab_array_var($_SESSION, 'container_names', array());
            $container_names = grab_array_var($inargs, 'containernames', $container_names_session);
            $container_names = array_values(array_filter($container_names));

            $network_names_session = grab_array_var($_SESSION, 'network_names', array());
            $network_names = grab_array_var($inargs, 'networknames', $network_names_session);
            $network_names = array_values(array_filter($network_names));

            $check_types_session = grab_array_var($_SESSION, 'check_type', array());
            $check_types = grab_array_var($inargs, 'checktype', $check_types_session);

            $cert_session = grab_array_var($_SESSION[$wizard_name], 'cert', '');
            $cert = grab_array_var($inargs, 'cert', $cert_session);
            $key_session = grab_array_var($_SESSION[$wizard_name], 'key', '');
            $key = grab_array_var($inargs, 'key', $key_session);
            $cacert_session = grab_array_var($_SESSION[$wizard_name], 'cacert', '');
            $cacert = grab_array_var($inargs, 'cacert', $cacert_session);

            /* Now lets check for errors before proceeding to CONFIGWIZARD_MODE_GETSTAGE2HTML*/
            /* Set the error count to 0 */
            $errors = 0;
            /* Create the error messages array */
            $errmsg = array();
            /* Check to see if the address field was blank */
            if (have_value($address) == false)
                /* address field was blank, add an error message to the array and increment the $errors count */
                $errmsg[$errors++] = _('No address specified.');
            if (have_value($baseurl) == false)
                $errmsg[$errors++] = _('No Docker API base URL specified.');
            
            if ($remote === "NCPA") {
                $ncpa_full_url = "https://" . $address . ":" . $port . "/api/plugins?token=" . $token;
                $cmd = '';
                $result = ncpa_plugin_check($ncpa_full_url, $cmd); // defined at end of file
                if($result > 0) {
                    if ($result === 1) {
                        $errmsg[$errors++] = _("cURL call to verify NCPA installation failed.");
                    } 
                    else if($result === 2) {
                        $errmsg[$errors++] = _("cURL call to NCPA port did not return JSON data.");
                    }
                    else if($result === 3) {
                        $errmsg[$errors++] = _("cURL call to NCPA port returned invalid JSON.");
                    }
                    else if($result === 4) {
                        $errmsg[$errors++] = _("check_docker.py not installed on remote machine.");
                    }
                    else {
                        $errmsg[$errors++] = _("cURL-related error: $result");
                    }
                }
            }

            $valid_list_types = array("networks", "containers", "all");
            
            if(empty($check_types)) {
                $errmsg[$errors++] = _('No check types specified.');
            }

            if ($list_type === "none") {
                $errmsg[$errors++] = _('Please choose a way to monitor your containers.');
            }
            else if (!in_array($list_type, $valid_list_types)) {
                $errmsg[$errors++] = _('Invalid container list type.');
            }
            else if($list_type === "containers") {
                if(empty($container_names)) {
                    $errmsg[$errors++] = _('No containers specified.');
                }
            }
            else if($list_type === "networks") {
                if(empty($network_names)) {
                    $errmsg[$errors++] = _('No networks specified.');
                }
            }
            
            /* Check to see if the $errors array contains errors */
            if ($errors>0){
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            else {
                $_SESSION[$wizard_name]['ip_address'] = $address;
                $_SESSION[$wizard_name]['socket'] = $socket;
                $_SESSION[$wizard_name]['baseurl'] = $baseurl;
                $_SESSION[$wizard_name]['port'] = $port;
                $_SESSION[$wizard_name]['token'] = $token;
                $_SESSION[$wizard_name]['list_type'] = $list_type;
                $_SESSION[$wizard_name]['container_names'] = $container_names;
                $_SESSION[$wizard_name]['network_names'] = $network_names;
                $_SESSION[$wizard_name]['check_types'] = $check_types;
                $_SESSION[$wizard_name]['remote'] = $remote;
                $_SESSION[$wizard_name]['cert'] = $cert;
                $_SESSION[$wizard_name]['key'] = $key;
                $_SESSION[$wizard_name]['cacert'] = $cacert;
            }
            
            /* The next line ends CONFIGWIZARD_MODE_VALIDATESTAGE1DATA */
            break;


            
        case CONFIGWIZARD_MODE_GETSTAGE2HTML:
            /* This defines $back variable, used for determining if the Back button */
            /* was clicked when the user was on CONFIGWIZARD_MODE_GETSTAGE3HTML */
            $back = encode_form_val(grab_array_var($_POST,'backButton',false),ENT_QUOTES);
            
            /* Continuing from CONFIGWIZARD_MODE_GETSTAGE1HTML */
            /* OR */
            /* Continuing from CONFIGWIZARD_MODE_VALIDATESTAGE2DATA due to user error */

            if (isset($_POST['ip_address'])) {
                /* Continuing from CONFIGWIZARD_MODE_GETSTAGE1HTML */
                /* The following line can be enabled for debugging purposes */
                //print_r('<br />Continuing from CONFIGWIZARD_MODE_VALIDATESTAGE1DATA<br />'); 
                
                /* Grab array variables from CONFIGWIZARD_MODE_GETSTAGE1HTML */
                $address = grab_array_var($inargs,'ip_address','');
            
                /* We will save these to the session variable array for later use */
                $_SESSION[$wizard_name]['ip_address'] = $address; 
                
                /* The following function queries DNS using $addess to see if it can determine a FQDN for the object */
                /* Obviously this is more relevant when the user provided an IP Address on CONFIGWIZARD_MODE_GETSTAGE1HTML */
                $hostname = gethostbyaddr($address);
                
                /* The following variables need to be defined to prevent coding errors */
                $list_type = "none";
                $container_names = array();
                $network_names = array();
                $check_types = array();
            }
            else {
                /* Continuing from CONFIGWIZARD_MODE_VALIDATESTAGE2DATA due to user error */
                /* The following line can be enabled for debugging purposes */
                //print_r('<br />Continuing from CONFIGWIZARD_MODE_VALIDATESTAGE2DATA due to user error<br />'); 
                
                $list_type_session = grab_array_var($_SESSION[$wizard_name],'list_type','');
                $list_type = grab_array_var($inargs,'list-type',$list_type_session);

                $container_names_session = grab_array_var($_SESSION[$wizard_name], 'container_names', array());
                $container_names = grab_array_var($inargs, 'containernames', $container_names_session);
                $container_names = array_values(array_filter($container_names));

                $network_names_session = grab_array_var($_SESSION[$wizard_name], 'network_names', array());
                $network_names = grab_array_var($inargs, 'network_names', $network_names_session);
                $network_names = array_values(array_filter($network_names));

                $check_types_session = grab_array_var($_SESSION[$wizard_name], 'check_types', array());
                $check_types = grab_array_var($inargs, 'check_types', $check_types_session);

                $hostname_session = grab_array_var($_SESSION[$wizard_name], 'hostname', '');
                $hostname = grab_array_var($inargs, 'hostname', $hostname_session);

                $address = grab_array_var($_SESSION[$wizard_name], 'ip_address', '');
            }
            // Old variables. These should not change for the rest of the wizard, so it shouldn't matter whether we're going forwards or backwards.
            $check_types = grab_array_var($_SESSION[$wizard_name], 'check_types', array());
            $list_type = grab_array_var($_SESSION[$wizard_name], 'list_type', "none");
            $container_names = grab_array_var($_SESSION[$wizard_name], 'container_names', array());
            $network_names = grab_array_var($_SESSION[$wizard_name], 'network_names', array());
            
            // New variable. This should also just kinda work.
            $serviceargs = grab_array_var($_SESSION[$wizard_name], 'serviceargs', array());
                
            /* Now we are creating the HTML for CONFIGWIZARD_MODE_GETSTAGE2HTML */ 

            // This redirects all 'printed' output to a buffer - see ob_get_clean() call at the end.
            ob_start(); 
            ?>

            <h5 class="ul"><?php echo _('Remote Host Details');?></h5>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td>
                        <label for="ip_address"><?php echo _('IP Address'); ?>:</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="ip_address" id="ip_address" value="<?php echo encode_form_val($address); ?>" class="textfield form-control" readonly>
                    </td>
                </tr>
                <tr>
                    <td>
                        <label for="hostname"><?php echo _('Host Name'); ?>:</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="hostname" id="hostname" value="<?php echo encode_form_val($hostname); ?>" class="textfield form-control">
                    </td>
                </tr>
            </table>
            <?php 
            $hide_exists = "hide";

            if (array_key_exists("exist", $check_types)) {
                $hide_exists = "";
            }
            $placeholders = array(
                'description' => _('Docker - Containers Exist'),
                'warning' => "50:",
                'critical' => '30:',
                'timeout' => '0'
            );
            foreach ($placeholders as $index => $value) {
                if (!array_key_exists('exists', $serviceargs) || !array_key_exists($index, $serviceargs['exist'])) {
                    $serviceargs['exist'][$index] = $value;
                }
            }

            ?>
            <div id="configure-exist" class="<?php echo $hide_exists; ?>">
                <h5 class="ul"><?php echo _('Existing Containers'); ?></h5>
                <table class="table table-no-border table-auto-width table-padded">
                    <tbody>
                        <tr>
                            <td></td>
                            <td>
                                <label for="serviceargs[exist][description]"><?php echo _('Service Description'); ?>:</label>
                                <input type="text" size="40" name="serviceargs[exist][description]" id="serviceargs[exist][description]" value="<?php echo encode_form_val($serviceargs['exist']['description']); ?>" class="textfield form-control">
                                <div class="subtext"><?php echo _('Warning: If you have another service with the same description, it will be overwritten by this service.'); ?></div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="count" class="select-cf-option"><?php echo _('Thresholds'); ?></label><br>
                                <?php echo sprintf(_('Enter a value like %1$s to alert if there are fewer than 10 containers.'), "'10:'"); ?><br/>
                                <?php echo sprintf(_('Enter a value like %1$s to alert if there are more than 10 containers.'), "'10'"); ?>
                                <div class="pad-t5">
                                    <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[exist][warning]" value="<?php echo encode_form_val($serviceargs['exist']['warning']); ?>" class="textfield form-control condensed"> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[exist][critical]" value="<?php echo encode_form_val($serviceargs['exist']['critical']); ?>" class="textfield form-control condensed">
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="timeout" class="select-cf-option"><?php echo _('Timeout'); ?></label><br/>
                                <?php echo _('Enter 0 to have the check never time out.'); ?>
                                <div class="pad-t5">
                                    <input type="text" size="1" name="serviceargs[exist][timeout]" value="<?php echo encode_form_val($serviceargs['exist']['timeout']); ?>" class="textfield form-control condensed">&nbsp;<?php echo _('seconds'); ?>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </div>

            <?php 

            $hide_running = "hide";
            if (array_key_exists("running", $check_types)) {
                $hide_running = "";
            }
            $placeholders = array(
                'description' => _('Docker - Containers Are Running'),
                'warning' => "50:",
                'critical' => '30:',
                'timeout' => '0',
                'bad' => 'on',
                'percentage' => '',
            );
            foreach ($placeholders as $index => $value) {
                if (!array_key_exists('running', $serviceargs) || !array_key_exists($index, $serviceargs['running'])) {
                    $serviceargs['running'][$index] = $value;
                }
            }

            ?>

            <div id="configure-running" class="<?php echo $hide_running; ?>">
                <h5 class="ul"><?php echo _('Running Containers'); ?></h5>
                <table class="table table-no-border table-auto-width table-padded">
                    <tbody>
                        <tr>
                            <td></td>
                            <td>
                                <label for="serviceargs[running][description]"><?php echo _('Service Description'); ?>:</label>
                                <input type="text" size="40" name="serviceargs[running][description]" id="serviceargs[running][description]" value="<?php echo encode_form_val($serviceargs['running']['description']);?>" class="textfield form-control">
                                <div class="subtext"><?php echo _('Warning: If you have another service with the same description, it will be overwritten by this service.'); ?></div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="count" class="select-cf-option"><?php echo _('Thresholds'); ?></label><br>
                                <?php echo sprintf(_('Enter a value like %1$s to alert if there are fewer than 10 running containers.'), "'10:'"); ?><br/>
                                <?php echo sprintf(_('Enter a value like %1$s to alert if there are more than 10 running containers.'), "'10'"); ?>
                                <div class="pad-t5">
                                    <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[running][warning]" value="<?php echo encode_form_val($serviceargs['running']['warning']); ?>" class="textfield form-control condensed"><div class="running-percentage hide"> % </div> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[running][critical]" value="<?php echo encode_form_val($serviceargs['running']['critical']); ?>" class="textfield form-control condensed"><div class="running-percentage hide"> % </div>
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="timeout" class="select-cf-option"><?php echo _('Timeout'); ?></label><br/>
                                <?php echo _('Enter 0 to have the check never time out.'); ?>
                                <div class="pad-t5">
                                    <input type="text" size="1" name="serviceargs[running][timeout]" value="<?php echo encode_form_val($serviceargs['running']['timeout']); ?>" class="textfield form-control condensed">&nbsp;<?php echo _('seconds'); ?>
                            </td>
                        </tr>
                        <tr>
                            <td>
                            <input type="checkbox" class="checkbox" id="running-bad" name="serviceargs[running][bad]" <?php echo is_checked($serviceargs['running']['bad']); ?>>
                            </td>
                            <td>
                                <label for="running-bad" class="select-cf-option"><?php echo _('List Non-Running Containers'); ?></label><br/>
                            </td>
                        </tr>
                        <tr>
                            <td>
                            <input type="checkbox" class="checkbox" id="running-percentage" name="serviceargs[running][percentage]" <?php echo is_checked($serviceargs['running']['percentage']); ?>>
                            </td>
                            <td>
                                <label for="running-percentage" class="select-cf-option"><?php echo _('Express thresholds as a percentage.'); ?></label><br/>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </div>

            <?php

            $hide_healthy = "hide";
            if (array_key_exists("healthy", $check_types)) {
                $hide_healthy = "";
            }
            $placeholders = array(
                'description' => _('Docker - Containers Are Healthy'),
                'warning' => "50:",
                'critical' => '30:',
                'timeout' => '0',
                'bad' => 'on',
                'percentage' => '',
                'no-healthcheck' => 'ignored',
            );
            foreach ($placeholders as $index => $value) {
                if (!array_key_exists('healthy', $serviceargs) || !array_key_exists($index, $serviceargs['healthy'])) {
                    $serviceargs['healthy'][$index] = $value;
                }
            }
            ?>


            <div id="configure-healthy" class="<?php echo $hide_healthy; ?>">
            
                <h5 class="ul"><?php echo _('Healthy Containers'); ?></h5>
                <table class="table table-no-border table-auto-width table-padded">
                    <tbody>
                        <tr>
                            <td></td>
                            <td>
                                <label for="serviceargs[healthy][description]"><?php echo _('Service Description'); ?>:</label>
                                <input type="text" size="40" name="serviceargs[healthy][description]" id="serviceargs[healthy][description]" value="<?php echo encode_form_val($serviceargs['healthy']['description']); ?>" class="textfield form-control">
                                <div class="subtext"><?php echo _('Warning: If you have another service with the same description, it will be overwritten by this service.'); ?></div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="count" class="select-cf-option"><?php echo _('Thresholds'); ?></label><br>
                                <?php echo sprintf(_('Enter a value like %1$s to alert if there are fewer than 10 healthy containers.'), '\'10:\''); ?><br/>
                                <?php echo sprintf(_('Enter a value like %1$s to alert if there are more than 10 healthy containers.'), '\'10\''); ?>
                                <div class="pad-t5">
                                    <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _("Warning Threshold"); ?>"></label> <input type="text" size="2" name="serviceargs[healthy][warning]" value="<?php echo encode_form_val($serviceargs['healthy']['warning']); ?>" class="textfield form-control condensed"><div class="healthy-percentage hide"> % </div> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _("Critical Threshold"); ?>"></label> <input type="text" size="2" name="serviceargs[healthy][critical]" value="<?php echo encode_form_val($serviceargs['healthy']['critical']); ?>" class="textfield form-control condensed"><div class="healthy-percentage hide"> % </div>
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="timeout" class="select-cf-option"><?php echo _('Timeout'); ?></label><br/>
                                <?php echo _('Enter 0 to have the check never time out.'); ?>
                                <div class="pad-t5">
                                    <input type="text" size="1" name="serviceargs[healthy][timeout]" value="<?php echo encode_form_val($serviceargs['healthy']['timeout']); ?>" class="textfield form-control condensed">&nbsp;<?php echo _('seconds'); ?>
                            </td>
                        </tr>
                        <tr>
                            <td>
                            <input type="checkbox" class="checkbox" id="healthy-bad" name="serviceargs[healthy][bad]" <?php echo is_checked($serviceargs['healthy']['bad']); ?>>
                            </td>
                            <td>
                                <label for="healthy-bad" class="select-cf-option"><?php echo _('List Unhealthy Containers'); ?></label><br/>
                            </td>
                        </tr>
                        <tr>
                            <td>
                            <input type="checkbox" class="checkbox" id="healthy-percentage" name="serviceargs[healthy][percentage]" <?php echo is_checked($serviceargs['healthy']['percentage']); ?>>
                            </td>
                            <td>
                                <label for="healthy-percentage" class="select-cf-option"><?php echo _('Express thresholds as a percentage.'); ?></label><br/>
                            </td>
                        </tr>
                        <tr>
                            <td>
                            </td>
                            <td>
                                <label for="healthy-nocheck" class="select-cf-option"><?php echo _('When a container has no health check, it should be'); ?></label><br/>
                                <select name="serviceargs[healthy][no-healthcheck]" id="no-healthcheck" class="form-control">
                                    <option value="ignored" <?php echo is_selected($serviceargs['healthy']['no-healthcheck'], 'ignored');?>><?php echo _(' ignored.'); ?></option>
                                    <option value="healthy" <?php echo is_selected($serviceargs['healthy']['no-healthcheck'], 'healthy');?>><?php echo _(' counted as a healthy container.'); ?></option>
                                    <option value="unhealthy" <?php echo is_selected($serviceargs['healthy']['no-healthcheck'], 'unhealthy');?>><?php echo _(' counted as an unhealthy container.'); ?></option> 
                                </select>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </div>

            <?php

            $hide_cpu = "hide";
            $hide_all_table = "hide";
            $hide_containers_table = "hide";
            $hide_networks_table = "hide";
            if ($list_type === "all") {
                $hide_all_table = "";
            }
            elseif ($list_type === "containers") {
                $hide_containers_table = "";
            }
            elseif ($list_type === "networks") {
                $hide_containers_table = "";
            }
            if (array_key_exists("cpu", $check_types)) {
                $hide_cpu = "";
            }

            $placeholders = array(
                'description' => _('Docker - Container CPU Usage'),
                'all' => array('warning' => "30", 'critical' => '50'),
                'networks' => array('warning' => array(), 'critical' => array()),
                'containers' => array('warning' => array(), 'critical' => array()),
                'timeout' => '0',
                'bad' => 'on',
                'percentage' => '',
                'networks-avg' => 'total',
                'aggregate' => "",
                'total' => "",
                'total-thresholds' => array('warning' => 50, 'critical' => 75),
                'avg' => 'on',
                'avg-thresholds' => array('warning' => 10, 'critical' => 20),
                'only' => '',
            );
            foreach ($placeholders as $index => $value) {
                if (!array_key_exists('cpu', $serviceargs) || !array_key_exists($index, $serviceargs['cpu'])) {
                    $serviceargs['cpu'][$index] = $value;
                }
            }
            ?>
            <div id="configure-cpu" class="<?php echo $hide_cpu; ?>">
                <h5 class="ul"><?php echo _('CPU Usage'); ?></h5>
                <div id="cpu-all-table" class="all-table <?php echo $hide_all_table; ?>">
                    <table class="table table-no-border table-auto-width table-padded">
                    <tbody>
                    <tr>
                        <td><input type="checkbox" style="visibility: hidden"></input></td>
                        <td>
                            <label for="count" class="select-cf-option"><?php echo _('Per-Container Thresholds'); ?></label><br>
                            <?php echo _('Each container the check finds will be checked against this threshold.'); ?><br/>
                            <div class="pad-t5">
                                <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _("Warning Threshold"); ?>"></label> <input type="text" size="2" name="serviceargs[cpu][all][warning][]" value="<?php echo encode_form_val($serviceargs['cpu']['all']['warning']); ?>" class="textfield form-control condensed"><div class="cpu-percentage hide"> % </div> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _("Critical Threshold"); ?>"></label> <input type="text" size="2" name="serviceargs[cpu][all][critical][]" value="<?php echo encode_form_val($serviceargs['cpu']['all']['critical']); ?>" class="textfield form-control condensed"><div class="cpu-percentage hide"> % </div>
                            </div>
                        </td>
                    </tr>
                    </tbody>
                    </table>
                </div>
                <div id="cpu-network-table" class="network-table <?php echo $hide_networks_table; ?>">
                    <table class="table table-no-border table-auto-width" style="margin: 0 0 10px 0;">
                        <tr> 
                            <th><?php echo _('Network Name/ID'); ?></th>
                            <th><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _("Warning Threshold"); ?>"><?php echo _('Warning'); ?></th>
                            <th><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _("Critical Threshold"); ?>"><?php echo _('Critical'); ?></th>
                        </tr>
                <?php 
                if(is_array($network_names)) {
                    foreach($network_names as $index => $network) {
                ?>
                        <tr>
                            <td>
                                <input type="text" size="40" name="networknames[<?php echo $index; ?>]" value="<?php echo encode_form_val($network); ?>" class="textfield form-control" disabled>
                            </td>
                            <td>
                                <input type="text" size="10" name="serviceargs[cpu][networks][warning][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['cpu']['networks']['warning'][$index]); ?>" class="textfield form-control">
                            </td>
                            <td>
                                <input type="text" size="10" name="serviceargs[cpu][networks][critical][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['cpu']['networks']['critical'][$index]); ?>" class="textfield form-control">
                            </td>
                        </tr>
                    <?php 
                        }
                    }
                    ?>

                    </table>
                    <table class="table table-condensed table-no-border table-auto-width table-padded" style="margin-bottom: 0px;">
                        <tbody>
                            <tr>
                                <td>
                                    <label for="networks-avg-or-tot" class="select-cf-option"><?php echo _('Networks use the '); ?>
                                        <select name="serviceargs[cpu][networks-avg]" class="form-control">
                                            <option value="total" <?php echo is_selected($serviceargs['cpu']['networks-avg'], 'total'); ?>><?php echo _(' total '); ?></option>
                                            <option value="average" <?php echo is_selected($serviceargs['cpu']['networks-avg'], 'average'); ?>><?php echo _('average'); ?></option>
                                        </select> 
                                        <?php echo _('CPU usage of their containers.'); ?></label>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                </div>
                <div id="cpu-container-table" class="container-table <?php echo ($list_type === "containers" ?  "" : "hide"); ?>">
                    <table class="table table-no-border table-auto-width" style="margin: 0 0 10px 0;">
                        <tr> 
                            <th><?php echo _('Container Name/ID'); ?></th>
                            <th><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _("Warning Threshold"); ?>"><?php echo _('Warning'); ?></th>
                            <th><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _("Critical Threshold"); ?>"><?php echo _('Critical'); ?></th>
                        </tr>
                <?php
                if(is_array($container_names)) {
                    foreach($container_names as $index => $container) {
                ?>
                        <tr>
                            <td class="first">
                                <input type="text" size="50" name="containernames[<?php echo $index; ?>]" value="<?php echo encode_form_val($container); ?>" class="textfield form-control" disabled>
                            </td>
                            <td>
                                <input type="text" size="10" name=serviceargs[cpu][containers][warning][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['cpu']['containers']['warning'][$index]); ?>" class="textfield form-control">
                            </td>
                            <td>
                                <input type="text" size="10" name=serviceargs[cpu][containers][critical][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['cpu']['containers']['critical'][$index]); ?>" class="textfield form-control">
                            </td>
                        </tr>
                <?php 
                    }
                }
                ?>
                    </table>
                </div>
                <table class="table table-no-border table-auto-width table-padded" style="margin-bottom: 0px;">
                    <tbody>
                        <tr>
                            <td></td>
                            <td>
                                <label for="serviceargs[cpu][description]"><?php echo _('Service Description'); ?>:</label>
                                <input type="text" size="40" name="serviceargs[cpu][description]" id="serviceargs[cpu][description]" value="<?php echo encode_form_val($serviceargs['cpu']['description']); ?>" class="textfield form-control">
                                <div class="subtext"><?php echo _('Warning: If you have another service with the same description, it will be overwritten by this service.'); ?></div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="timeout" class="select-cf-option"><?php echo _('Timeout'); ?></label><br/>
                                <?php echo _('Enter 0 to have the check never time out.'); ?>
                                <div class="pad-t5">
                                    <input type="text" size="1" name="serviceargs[cpu][timeout]" value="<?php echo encode_form_val($serviceargs['cpu']['timeout']); ?>" class="textfield form-control condensed">&nbsp;<?php echo _('seconds'); ?>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <input type="checkbox" class="checkbox" id="cpu-agg" name="serviceargs[cpu][aggregate]" <?php echo is_checked($serviceargs['cpu']['aggregate']); ?>>
                            </td>
                            <td>
                                <label for="cpu-agg" class="select-cf-option"><?php echo _('Use aggregate statistics'); ?></label>
                            </td>
                        </tr>
                    </tbody>
                </table>
                <table class="table table-no-border table-auto-width table-padded hide aggregate-options" id="cpu-aggregate-options">
                    <tbody>
                        <tr>
                            <td><input type="checkbox" class="checkbox hide" id="" name=""></td>
                            <td>
                                <input type="checkbox" class="checkbox" id="cpu-total" name="serviceargs[cpu][total]" <?php echo is_checked($serviceargs['cpu']['total']); ?>>
                            </td>
                            <td>
                                <label for="cpu-total" class="select-cf-option"><?php echo _('Check the total CPU usage.'); ?></label>
                                <div class="pad-t5">
                                    <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[cpu][total-thresholds][warning]" value="<?php echo encode_form_val($serviceargs['cpu']['total-thresholds']['warning']); ?>" class="textfield form-control condensed"> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[cpu][total-thresholds][critical]" value="<?php echo encode_form_val($serviceargs['cpu']['total-thresholds']['critical']); ?>" class="textfield form-control condensed">
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <input type="checkbox" class="checkbox" id="cpu-avg" name="serviceargs[cpu][avg]" <?php echo is_checked($serviceargs['cpu']['avg']); ?>>
                            </td>
                            <td>
                                <label for="cpu-avg" class="select-cf-option"><?php echo _('Check the average (mean) CPU usage.'); ?></label>
                                <div class="pad-t5">
                                    <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[cpu][avg-thresholds][warning]" value="<?php echo encode_form_val($serviceargs['cpu']['avg-thresholds']['warning']); ?>" class="textfield form-control condensed"> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[cpu][avg-thresholds][critical]" value="<?php echo encode_form_val($serviceargs['cpu']['avg-thresholds']['critical']); ?>" class="textfield form-control condensed">
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <input type="checkbox" class="checkbox" id="cpu-only" name="serviceargs[cpu][only]" <?php echo is_checked($serviceargs['cpu']['only']); ?>>
                            </td>
                            <td>
                                <label for="cpu-only" class="select-cf-option"><?php echo _('Use only aggregate usage values (disables individual thresholds).'); ?></label>
                            </td>
                        </tr>
                    </tbody>
                </table>

                <script>
                    $(document).ready(function () {
                        if ($("#cpu-agg").prop("checked")) {
                            $("#cpu-aggregate-options").show();
                        }
                        if ($("#cpu-only").prop("checked")) {
                            $("#cpu-container-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                            $("#cpu-network-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                        }
                    });
                    $("#cpu-agg").on("click", function () {
                        if ($(this).prop("checked")) {
                            $("#cpu-aggregate-options").show();
                        }
                        else {
                            $("#cpu-aggregate-options").hide();
                        }
                    });
                    $("#cpu-only").on("click", function () {
                        if ($("#cpu-only").prop("checked")) {
                            $("#cpu-container-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                            $("#cpu-network-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                        }
                        else {
                            $("#cpu-container-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", false);
                            $("#cpu-network-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", false);
                        }
                    });
                </script>

            </div>
            <?php 

            $hide_memory = "hide";

            if (array_key_exists("memory", $check_types)) {
                $hide_memory = "";
            }
            $placeholders = array(
                'description' => _('Docker - Container Memory Usage'),
                'all' => array('warning' => "30", 'critical' => '50'),
                'networks' => array('warning' => array(), 'critical' => array()),
                'containers' => array('warning' => array(), 'critical' => array()),
                'timeout' => '0',
                'bad' => 'on',
                'percentage' => '',
                'networks-avg' => 'total',
                'aggregate' => "",
                'total' => "",
                'total-thresholds' => array('warning' => 50, 'critical' => 75),
                'avg' => 'on',
                'avg-thresholds' => array('warning' => 10, 'critical' => 20),
                'only' => '',
            );
            foreach ($placeholders as $index => $value) {
                if (!array_key_exists('memory', $serviceargs) || !array_key_exists($index, $serviceargs['memory'])) {
                    $serviceargs['memory'][$index] = $value;
                }
            }
            ?>

            <div id="configure-memory" class="<?php echo $hide_memory; ?>">
                <h5 class="ul"><?php echo _('Memory Usage'); ?></h5>
                <div id="memory-all-table" class="all-table <?php echo $hide_all_table; ?>">
                    <table class="table table-no-border table-auto-width table-padded">
                    <tbody>
                    <tr>
                        <td><input type="checkbox" style="visibility: hidden"></input></td>
                        <td>
                            <label for="count" class="select-cf-option"><?php echo _('Per-Container Thresholds'); ?></label><br>
                            <?php echo _('Each container the check finds will be checked against this threshold.'); ?><br/>
                            <div class="pad-t5">
                                <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _("Warning Threshold"); ?>"></label> <input type="text" size="2" name="serviceargs[memory][all][warning][]" value="<?php echo $serviceargs['memory']['all']['warning']; ?>" class="textfield form-control condensed"><div class="memory-percentage hide"> % </div> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _("Critical Threshold"); ?>"></label> <input type="text" size="2" name="serviceargs[memory][all][critical][]" value="<?php echo encode_form_val($serviceargs['memory']['all']['critical']); ?>" class="textfield form-control condensed"><div class="memory-percentage hide"> % </div>
                            </div>
                        </td>
                    </tr>
                    </tbody>
                    </table>
                </div>
                <div id="memory-network-table" class="network-table <?php echo $hide_networks_table; ?>">
                    <table class="table table-no-border table-auto-width" style="margin: 0 0 10px 0;">
                        <tr> 
                            <th><?php echo _('Network Name/ID'); ?></th>
                            <th><div><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"><span><?php echo _('Warning'); ?></span></div></th>
                            <th><div><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"><span><?php echo _('Critical'); ?></span></div></th>
                        </tr>
                <?php
                if(is_array($network_names)) {
                    foreach($network_names as $index => $network) {
                ?>
                        <tr>
                            <td>
                                <input type="text" size="40" name="networknames[<?php echo $index; ?>]" value="<?php echo encode_form_val($network); ?>" class="textfield form-control condensed" disabled>
                            </td>
                            <td>
                                <input type="text" size="10" name="serviceargs[memory][networks][warning][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['memory']['networks']['warning'][$index]); ?>" class="textfield form-control condensed">
                            </td>
                            <td>
                                <input type="text" size="10" name="serviceargs[memory][networks][critical][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['memory']['networks']['critical'][$index]); ?>" class="textfield form-control condensed">
                            </td>
                        </tr>
                <?php
                    }
                }
                ?>

                    </table>
                    <table class="table table-no-border table-auto-width table-padded" style="margin-bottom: 0px;">
                        <tbody>
                            <th>
                                <label for="networks-avg-or-tot" class="select-cf-option"><?php echo _('Networks use the '); ?>
                                    <select name="serviceargs[memory][networks-avg]" class="form-control">
                                        <option value="total" <?php echo is_selected($serviceargs['memory']['networks-avg'], 'total'); ?>><?php echo _(' total '); ?></option>
                                        <option value="average" <?php echo is_selected($serviceargs['memory']['networks-avg'], 'average'); ?>><?php echo _(' average '); ?></option>
                                    </select> 
                                    <?php echo _('memory usage of their containers.'); ?></label>
                            </th>
                        </tbody>
                    </table>
                </div>
                <div id="memory-container-table" class="container-table <?php echo $hide_containers_table; ?>">
                    <table class="table table-no-border table-auto-width" style="margin: 0 0 10px 0;">
                        <tr> 
                            <th><?php echo _('Container Name/ID'); ?></th>
                            <th><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"><?php echo _('Warning'); ?></th>
                            <th><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"><?php echo _('Critical'); ?></th>
                        </tr>
                <?php 
                if(is_array($container_names)) {
                    foreach($container_names as $index => $container) {
                ?>
                        <tr>
                            <td>
                                <input type="text" size="50" name="containernames[<?php echo $index; ?>]" value="<?php echo encode_form_val($container); ?>" class="textfield form-control" disabled>
                                <input type="hidden" name="containernames[<?php echo $index; ?>]" value="<?php echo encode_form_val($container); ?>">
                            </td>
                            <td>
                                <input type="text" size="10" name="serviceargs[memory][containers][warning][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['memory']['containers']['warning'][$index]); ?>" class="textfield form-control">
                            </td>
                            <td>
                                <input type="text" size="10" name="serviceargs[memory][containers][critical][<?php echo $index; ?>]" value="<?php echo encode_form_val($serviceargs['memory']['containers']['critical'][$index]); ?>" class="textfield form-control">
                            </td>
                        </tr>
                <?php 
                    }
                }
                ?>
                    </table>
                </div>
                <table class="table table-no-border table-auto-width table-padded" style="margin-bottom: 0px;">
                    <tbody>
                        <tr>
                            <td></td>
                            <td>
                                <label for="serviceargs[memory][description]"><?php echo _('Service Description'); ?>:</label>
                                <input type="text" size="40" name="serviceargs[memory][description]" id="serviceargs[memory][description]" value="<?php echo encode_form_val($serviceargs['memory']['description']); ?>" class="textfield form-control">
                                <div class="subtext"><?php echo _('Warning: If you have another service with the same description, it will be overwritten by this service.'); ?></div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <label for="timeout" class="select-cf-option"><?php echo _('Timeout'); ?></label><br/>
                                <?php echo _('Enter 0 to have the check never time out.'); ?>
                                <div class="pad-t5">
                                    <input type="text" size="1" name="serviceargs[memory][timeout]" value="<?php echo encode_form_val($serviceargs['memory']['timeout']); ?>" class="textfield form-control condensed">&nbsp;seconds
                            </td>
                        </tr>
                        <tr>
                            <td>
                            </td>
                            <td>
                                <label for="mem-bytes-or-percent" class="select-cf-option"><?php echo _('Express a container\'s memory usage '); ?> 
                                    <select name="serviceargs[memory][percentage]" class="form-control">
                                        <option value="bytes" <?php echo is_selected($serviceargs['memory']['percentage'], 'bytes'); ?>><?php echo _('in bytes '); ?></option>
                                        <option value="kibibytes" <?php echo is_selected($serviceargs['memory']['percentage'], 'kibibytes'); ?>><?php echo _('in KiB '); ?></option>
                                        <option value="mebibytes" <?php echo is_selected($serviceargs['memory']['percentage'], 'mebibytes'); ?>><?php echo _('in MiB '); ?></option>
                                        <option value="gibibytes" <?php echo is_selected($serviceargs['memory']['percentage'], 'gibibytes'); ?>><?php echo _('in GiB '); ?></option>
                                        <option value="percentage" <?php echo is_selected($serviceargs['memory']['percentage'], 'percentage'); ?>><?php echo _('as a percent of its limit.'); ?></option>
                                    </select></label>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <input type="checkbox" class="checkbox" id="memory-agg" name="serviceargs[memory][aggregate]" <?php echo is_checked($serviceargs['memory']['aggregate']); ?>>
                            </td>
                            <td>
                                <label for="memory-agg" class="select-cf-option"><?php echo _('Use aggregate statistics'); ?></label>
                            </td>
                        </tr>
                    </tbody>
                </table>
                <table class="table table-no-border table-auto-width table-padded hide aggregate-options" id="memory-aggregate-options">
                    <tbody>
                        <tr>
                            <td><input type="checkbox" class="checkbox hide" id="" name=""></td>
                            <td>
                                <input type="checkbox" class="checkbox" id="memory-total" name="serviceargs[memory][total]" <?php echo is_checked($serviceargs['memory']['total']); ?>>
                            </td>
                            <td>
                                <label for="memory-total" class="select-cf-option"><?php echo _('Check the total memory usage.'); ?></label>
                                <div class="pad-t5">
                                    <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[memory][total-thresholds][warning]" value="<?php echo encode_form_val($serviceargs['memory']['total-thresholds']['warning']); ?>" class="textfield form-control condensed"> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[memory][total-thresholds][critical]" value="<?php echo encode_form_val($serviceargs['memory']['total-thresholds']['critical']); ?>" class="textfield form-control condensed">
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <input type="checkbox" class="checkbox" id="memory-avg" name="serviceargs[memory][avg]" <?php echo is_checked($serviceargs['memory']['avg']); ?>>
                            </td>
                            <td>
                                <label for="memory-avg" class="select-cf-option"><?php echo _('Check the average (mean) memory usage.'); ?></label>
                                <div class="pad-t5">
                                    <label><img src="/nagiosxi/images/error.png" class="tt-bind" title="<?php echo _('Warning Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[memory][avg-thresholds][warning]" value="<?php echo encode_form_val($serviceargs['memory']['avg-thresholds']['warning']); ?>" class="textfield form-control condensed"> &nbsp; <label><img src="/nagiosxi/images/critical_small.png" class="tt-bind" title="<?php echo _('Critical Threshold'); ?>"></label> <input type="text" size="2" name="serviceargs[memory][avg-thresholds][critical]" value="<?php echo encode_form_val($serviceargs['memory']['avg-thresholds']['critical']); ?>" class="textfield form-control condensed">
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td></td>
                            <td>
                                <input type="checkbox" class="checkbox" id="memory-only" name="serviceargs[memory][only]" <?php echo is_checked($serviceargs['memory']['only']); ?>>
                            </td>
                            <td>
                                <label for="memory-only" class="select-cf-option"><?php echo _('Use only aggregate usage values (disables individual thresholds).'); ?></label>
                            </td>
                        </tr>
                    </tbody>
                </table>

                <script>
                    $(document).ready(function () {
                        if ($("#memory-agg").prop("checked")) {
                            $("#memory-aggregate-options").show();
                        }
                        if ($("#memory-only").prop("checked")) {
                            $("#memory-container-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                            $("#memory-network-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                        }
                    });
                    $("#memory-agg").on("click", function () {
                        $("#memory-aggregate-options").toggle();
                    });
                    $("#memory-only").on("click", function () {
                        if ($("#memory-only").prop("checked")) {
                            $("#memory-container-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                            $("#memory-network-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", true);
                        }
                        else {
                            $("#memory-container-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", false);
                            $("#memory-network-table > table > tbody > tr > td:not(:first-child)").children().prop("disabled", false);
                        }
                    });
                </script>

            </div>

            <?php 
            $output = ob_get_clean();

            /* The quote and semicolon ends HTML for CONFIGWIZARD_MODE_GETSTAGE2HTML */ 
            /* $output will be passed back to XI framework and rendered to browser */
            /* The next line ends CONFIGWIZARD_MODE_GETSTAGE2HTML */
            break;

        /* Form validation for CONFIGWIZARD_MODE_GETSTAGE2HTML */ 
        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:
            /* This defines $back variable, used for determining if the Back button */
            /* was clicked when the user was on CONFIGWIZARD_MODE_GETSTAGE4HTML */
            $back = array_key_exists("backButton", $_POST);         
            /* If the user came back from CONFIGWIZARD_MODE_GETSTAGE3HTML then we don't need to revalidate and check for errors */
            if ($back) break;
            

            $hostname_session = grab_array_var($_SESSION, 'hostname', '');
            $hostname = grab_array_var($inargs, 'hostname', $hostname_session);

            $remote = grab_array_var($_SESSION, 'remote', '');

            $serviceargs = grab_array_var($_SESSION[$wizard_name], 'serviceargs', array());
            $serviceargs = grab_array_var($inargs, 'serviceargs', $serviceargs);

            $containernames = grab_array_var($_SESSION[$wizard_name], 'containernames', array());
            $containernames = grab_array_var($inargs, 'containernames');

            if(!have_value($hostname)) {
                $errmsg[$errors++] = _("No host name specified.");
            }
            elseif(!is_valid_host_name($hostname)) {
                $errmsg[$errors++] = _("Invalid host name.");
            }
            
            $_SESSION[$wizard_name]['hostname'] = $hostname;

            if (empty($serviceargs)) {
                $errmsg[$errors++] = _("serviceargs is empty. This error should never be reached. Please contact the config wizard maintainer.");
            }

            $check_types = grab_array_var($_SESSION[$wizard_name], 'check_types', array());

            foreach ($check_types as $check_type => $on) {
                // All checks need service descriptions.
                if (!is_valid_service_name($serviceargs[$check_type]['description'])) {
                    $errmsg[$errors++] = _('The check for ') . $check_type . _(' has an invalid service description') . ' (' . encode_form_val($serviceargs[$check_type]['description']) . ')';
                }
                $serviceargs[$check_type]['timeout'] = intval($serviceargs[$check_type]['timeout']);

                if ($check_type === 'cpu' || $check_type === 'memory' && !$serviceargs[$check_type]['only']) {
                    // If we're not *only* using aggregate statistics, we need to check that thresholds were entered for each container.

                    foreach ($serviceargs[$check_type]['containers']['warning'] as $i => $value) {
                        if (empty($value)) {
                            $errmsg[$errors++] = sprintf(_('Container \'%1$s\' does not have a WARNING value for its %2$s check'), $containernames[$i], strtoupper($check_type));
                        }
                    }

                    foreach ($serviceargs[$check_type]['containers']['critical'] as $i => $value) {
                        if (empty($value)) {
                            $errmsg[$errors++] = sprintf(_('Container \'%1$s\' does not have a CRITICAL value for its %2$s check'), $containernames[$i], strtoupper($check_type));
                        }
                    }
                }
            }


            if ($errors>0){
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
                if (!array_key_exists('serviceargs', $_SESSION[$wizard_name])) {
                    // If we've never assigned serviceargs to SESSION, it's fine to overwrite even if they're not valid.
                    $_SESSION[$wizard_name]['serviceargs'] = $serviceargs;
                    $_SESSION[$wizard_name]['containernames'] = $containernames;
                }
            }
            else {
                $_SESSION[$wizard_name]['serviceargs'] = $serviceargs;
                $_SESSION[$wizard_name]['containernames'] = $containernames;
            }

            /* The next line ends CONFIGWIZARD_MODE_VALIDATESTAGE2DATA */
            break;

        
        case CONFIGWIZARD_MODE_GETSTAGE3HTML:
            $back = array_key_exists('backButton', $_POST);


            $output = '
            ';
            /* The next line ends CONFIGWIZARD_MODE_GETSTAGE3HTML */
            break;
            
            
        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:

            $back = array_key_exists("backButton", $_POST);
            if($back) {
                break;
            }

            if($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;
    
        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:
            $output = '';
            break;
        
        
        case CONFIGWIZARD_MODE_GETOBJECTS:
            $remote = $_SESSION[$wizard_name]['remote'];
            $address = $_SESSION[$wizard_name]['ip_address'];
            $hostname = $_SESSION[$wizard_name]['hostname'];
            $socket = $_SESSION[$wizard_name]['socket'];
            $baseurl = $_SESSION[$wizard_name]['baseurl'];

            $token = escapeshellarg($_SESSION[$wizard_name]['token']);
            $port = $_SESSION[$wizard_name]['port'];

            $list_type = $_SESSION[$wizard_name]['list_type'];
            $container_names = $_SESSION[$wizard_name]['container_names'];
            $network_names = $_SESSION[$wizard_name]['network_names'];
            
            $check_types = $_SESSION[$wizard_name]['check_types'];
            $serviceargs = $_SESSION[$wizard_name]['serviceargs'];

            $cert = $_SESSION[$wizard_name]['cert'];
            $key = $_SESSION[$wizard_name]['key'];
            $cacert = $_SESSION[$wizard_name]['cacert'];

            $objs = array();

            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_linuxserver_host",
                    "host_name" => $hostname,
                    'icon_image' => 'docker.png',
                    "address" => $address,
                    "_xiwizard" => $wizard_name,
                );
            }

            foreach ($check_types as $check_type => $on) {
                $options_string = array();
                $sa = $serviceargs[$check_type];
                $options_string[] = "-H $baseurl";
                if(!empty($socket))
                    $options_string[] = '-s ' . $socket;
                $options_string[] = "--check-type 'containers_" . $check_type . "'";

                if ($list_type === "containers") {
                    $to_monitor = "-C '" . implode(",", $container_names) . "'";
                }
                elseif ($list_type === "networks") {
                    $to_monitor = "-N '" . implode(",", $network_names) . "'";
                }
                else {
                    $list_type = 'all';
                    $to_monitor = "--all";
                }

                if($remote === "api") {
                    if(!empty($cert))
                        $options_string[] = "--cert ". $cert;
                    if(!empty($key))
                        $options_string[] = "--key ". $key;
                    if(!empty($cacert))
                        $options_string[] = "--cacert ". $cacert;
                }

                $options_string[] = $to_monitor;
                $options_string[] = "-t " . $sa['timeout'];

                if(array_key_exists("networks-avg", $sa)) {
                    $options_string[] = "--networks-use-avg";
                }
                if($check_type === "healthy") {
                    if ($sa["no-healthcheck"] === "ignored") {
                        $options_string[] = "--ignore-no-healthcheck";
                    }
                    elseif($sa["no-healthcheck"] === "healthy") {
                        $options_string[] = "--no-check-is-healthy";
                    }
                }
                if (in_array($check_type, array("running", "healthy"))) {
                    if(array_key_exists("percentage", $sa) && $sa["percentage"] === "on")
                        $options_string[] = "--percentage";
                }

                if (in_array($check_type, array("running", "healthy"))) {
                    if (array_key_exists("bad", $sa) && $sa["bad"] === "on")
                        $options_string[] = "-l";
                }

                if (in_array($check_type, array("exist", "running", "healthy"))) {
                    $options_string[] = "-w '" . $sa['warning'] . "'";
                    $options_string[] = "-c '" . $sa['critical'] . "'";
                }
                else {
                    $warning = array();
                    $critical = array();
                    if(array_key_exists("aggregate", $sa)) {
                        if(array_key_exists("total", $sa)) {
                            $options_string[] = "--total-usage ";
                            $warning[] = $sa['total-thresholds']['warning'];
                            $critical[] = $sa['total-thresholds']['critical'];
                        }
                        if(array_key_exists("avg", $sa)) {
                            $options_string[] = "--total-average ";
                            $warning[] = $sa['avg-thresholds']['warning'];
                            $critical[] = $sa['avg-thresholds']['critical'];
                        }
                        if(!array_key_exists("only", $sa)) {
                            $warning = array_merge($warning, $sa[$list_type]['warning']);
                            $critical = array_merge($critical, $sa[$list_type]['critical']);
                        }
                    }
                    else {
                        $warning = $sa[$list_type]['warning'];
                        $critical = $sa[$list_type]['critical'];
                    }
                    if($check_type === "memory") {
                        if(array_key_exists("percentage", $sa)) {
                            switch($sa["percentage"]) {
                                case 'kibibytes':
                                    $options_string[] = "--memory-unit KiB";
                                    break;
                                case 'mebibytes':
                                    $options_string[] = "--memory-unit MiB";
                                    break;
                                case 'gibibytes':
                                    $options_string[] = "--memory-unit GiB";
                                    break;
                                case 'percentage':
                                    $options_string[] = "--percentage";
                                    break;
                                case 'bytes':
                                default:
                                    break;
                            }
                        }
                    }
                    $warning = implode(",", $warning);
                    $critical = implode(",", $critical);
                    $options_string[] = "-w '" . $warning . "'";
                    $options_string[] = "-c '" . $critical . "'";
                }

                if ($remote === "api") {

                    $all_options = implode($options_string, " "); 
                    $full_check_command = "check_docker!" . $all_options;
                }
                else {

                    $all_options = "-t " . $token . " -P ". $port . " -M plugins/check_docker.py -q \"";
                    foreach ($options_string as $ind => $option) {
                        $options_string[$ind] = "args=". $option;
                    }

                    $options_string = implode($options_string, ",");
                    $all_options .= $options_string . "\"";
                    $full_check_command = 'check_xi_ncpa!' . $all_options;
                }

                $objs[] = array(
                    'type'                  => OBJECTTYPE_SERVICE,
                    'host_name'             => $hostname,
                    'service_description'   => $sa['description'],
                    'icon_image'            => 'docker.png',
                    'check_command'         => $full_check_command,
                    '_xiwizard'             => $wizard_name,
                );

            }
            
            /* Return the object definitions to the wizard */
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;
            
            /* clear the session variables for this wizard run */
            unset ($_SESSION[$wizard_name]); 
            
            /* The next line ends CONFIGWIZARD_MODE_GETOBJECTS */
            break;
            
        default:
            break;          
    };   
    return $output;
}

function ncpa_plugin_check($url, &$cmd) {
    $url = escapeshellarg($url);
    $cmd = "curl " . $url . " -g -f -k --connect-timeout 10";

    $ret = 0;
    exec($cmd, $data, $ret);
    $data = implode("", $data);

    if ($ret) {
        return 1; // cURL-specific error
    }

    $data = json_decode($data, true);

    if (!$data) {
        return 2; // Not JSON data
    }

    if(!array_key_exists('plugins', $data)) {
        return 3; // Not the correct json data
    }

    $found_it = 4; // If not changed, plugin not installed.
    foreach($data['plugins'] as $plugin) {
        if ($plugin === "check_docker.py") {
            $found_it = 0; // Everything is good.
            break;
        }
    }

    return $found_it;
}
